unit MainForm;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, B42EncryptedFileStream, StdCtrls, DB, DBClient, Grids, DBGrids,
  ZLib, DBCtrls;

type
  TForm1 = class(TForm)
    ClientDataSet1: TClientDataSet;
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    btnSave: TButton;
    btnLoad: TButton;
    btnSaveX: TButton;
    btnLoadX: TButton;
    btnSaveZ: TButton;
    btnLoadZ: TButton;
    btnSaveZipX: TButton;
    btnLoadZipX: TButton;
    DBImage1: TDBImage;
    ClientDataSet1SpeciesNo: TFloatField;
    ClientDataSet1Category: TStringField;
    ClientDataSet1Common_Name: TStringField;
    ClientDataSet1SpeciesName: TStringField;
    ClientDataSet1Lengthcm: TFloatField;
    ClientDataSet1Length_In: TFloatField;
    ClientDataSet1Notes: TMemoField;
    ClientDataSet1Graphic: TGraphicField;
    DBMemo1: TDBMemo;
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure btnSaveXClick(Sender: TObject);
    procedure btnLoadXClick(Sender: TObject);
    procedure btnSaveZClick(Sender: TObject);
    procedure btnLoadZClick(Sender: TObject);
    procedure btnSaveZipXClick(Sender: TObject);
    procedure btnLoadZipXClick(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation
{$R *.dfm}

const
  _FileName = 'biolife';
  ExtNormalCDS = '.cds';
  ExtEncrypted = '.cdx';
  ExtCompressed= '.cdz';
  ExtComprEncr = '.cdy';

function GetFileSize(const FileName: String): Int64;
var
  SearchRec: TSearchRec;
begin
  Result := -1;
  if FindFirst(FileName, 0, SearchRec) = 0 then
    Result := SearchRec.Size;
  FindClose(SearchRec)
end;

//--- saving normal data

procedure TForm1.btnSaveClick(Sender: TObject);
var
  FileStream: TFileStream;
begin
  (Sender as TButton).Enabled := False;
  FileStream := TFileStream.Create(_FileName + ExtNormalCDS, fmCreate);
  try
    ClientDataSet1.SaveToStream(FileStream, dfBinary)
  finally
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

procedure TForm1.btnLoadClick(Sender: TObject);
var
  FileStream: TFileStream;
begin
  (Sender as TButton).Enabled := False;
  if ClientDataSet1.Active then ClientDataSet1.EmptyDataSet; // clear contents
  FileStream := TFileStream.Create(_FileName + ExtNormalCDS, fmOpenRead);
  try
    ClientDataSet1.LoadFromStream(FileStream)
  finally
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

//--- saving encrypted data

procedure TForm1.btnSaveXClick(Sender: TObject);
var
  FileStream: TB42EncryptedFileStream;
begin
  (Sender as TButton).Enabled := False;
  FileStream := TB42EncryptedFileStream.Create(_FileName + ExtEncrypted, fmCreate);
  try
    ClientDataSet1.SaveToStream(FileStream, dfBinary)
  finally
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

procedure TForm1.btnLoadXClick(Sender: TObject);
var
  FileStream: TB42EncryptedFileStream;
begin
  (Sender as TButton).Enabled := False;
  if ClientDataSet1.Active then ClientDataSet1.EmptyDataSet; // clear contents
  FileStream := TB42EncryptedFileStream.Create(_FileName + ExtEncrypted, fmOpenRead);
  try
    ClientDataSet1.LoadFromStream(FileStream)
  finally
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

//--- saving ZIP compressed data

procedure TForm1.btnSaveZClick(Sender: TObject);
var
  FileStream: TFileStream;
  ZipStream: TCompressionStream;
begin
  (Sender as TButton).Enabled := False;
  FileStream := TFileStream.Create(_FileName + ExtCompressed, fmCreate);
  ZipStream := TCompressionStream.Create(clMax, FileStream);
  try
    ClientDataSet1.SaveToStream(ZipStream, dfBinary)
  finally
    ZipStream.Free;
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

procedure TForm1.btnLoadZClick(Sender: TObject);
var
  FileStream: TFileStream;
  ZipStream: TDecompressionStream;
  MemStream: TMemoryStream;
begin
  (Sender as TButton).Enabled := False;
  if ClientDataSet1.Active then ClientDataSet1.EmptyDataSet; // clear contents
  FileStream := TFileStream.Create(_FileName + ExtCompressed, fmOpenRead);
  FileStream.Position := 0;
  ZipStream := TDecompressionStream.Create(FileStream);
  MemStream := TMemoryStream.Create;
  MemStream.CopyFrom(ZipStream, GetFileSize(_FileName + ExtNormalCDS));
  MemStream.Position := 0;
  try
    ClientDataSet1.LoadFromStream(MemStream)
  finally
    MemStream.Free;
    ZipStream.Free;
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

//--- saving ZIP and encrypted data

procedure TForm1.btnSaveZipXClick(Sender: TObject);
var
  FileStream: TB42EncryptedFileStream;
  ZipStream: TCompressionStream;
begin
  (Sender as TButton).Enabled := False;
  FileStream := TB42EncryptedFileStream.Create(_FileName + ExtComprEncr, fmCreate);
  ZipStream := TCompressionStream.Create(clMax, FileStream);
  try
    ClientDataSet1.SaveToStream(ZipStream, dfBinary)
  finally
    ZipStream.Free;
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

procedure TForm1.btnLoadZipXClick(Sender: TObject);
var
  FileStream: TB42EncryptedFileStream;
  ZipStream: TDecompressionStream;
  MemStream: TMemoryStream;
begin
  (Sender as TButton).Enabled := False;
  if ClientDataSet1.Active then ClientDataSet1.EmptyDataSet; // clear contents
  FileStream := TB42EncryptedFileStream.Create(_FileName + ExtComprEncr, fmOpenRead);
  FileStream.Position := 0;
  ZipStream := TDecompressionStream.Create(FileStream);
  MemStream := TMemoryStream.Create;
  MemStream.CopyFrom(ZipStream, GetFileSize(_FileName + ExtNormalCDS));
  MemStream.Position := 0;
  try
    ClientDataSet1.LoadFromStream(MemStream)
  finally
    MemStream.Free;
    ZipStream.Free;
    FileStream.Free;
    (Sender as TButton).Enabled := True
  end
end;

end.
